<?php
/*********************************************************************\
| (c)2017 codemonster                                                 |
|---------------------------------------------------------------------|
| This program is free software; you can redistribute it and/or       |
| modify it under the terms of the GNU General Public License         |
| as published by the Free Software Foundation; either version 2      |
| of the License, or (at your option) any later version.              |
|                                                                     |
| This program is distributed in the hope that it will be useful,     |
| but WITHOUT ANY WARRANTY; without even the implied warranty of      |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       |
| GNU General Public License for more details.                        |
|                                                                     |
| You should have received a copy of the GNU General Public License   |
| along with this program; If not, see <http://www.gnu.org/licenses/> |
\*********************************************************************/
?>
<?php
class CodemonsterDlmSearchZooqle {
		private $url_domain = 'https://zooqle.com';
		private $surl = '/search';
		private $qurl = '?pg=1&q=%s&v=t&s=ns&sd=d';
		private $aurl = '/api/media/%s';

		public function __construct()
		{
			$this->qurl=$this->url_domain.$this->surl.$this->qurl;
			$this->aurl=$this->url_domain.$this->aurl;
		}

		public function prepare($curl, $query)
		{
			$this->configureCurl($curl, sprintf($this->qurl, urlencode($query)));
		}

		public function parse($plugin, $response) {
			$this->processResultList($plugin, $response);

			try{
				$urlNavigationArray = array();
				$regexGetPageUrl = 'active><a href="(\?pg.*sd=d)"';
				if(preg_match_all("/$regexGetPageUrl/sU", $response, $pageUrls, PREG_SET_ORDER))
				{
					foreach ($pageUrls as $pageUrl)
					{
						parse_str(html_entity_decode($pageUrl[1]), $urlNavigationArray);
					}
				}

				for ($i = 2; $i <= 1; $i++) // Not used due to api limit ... DLM is too slow
				{
					if(array_key_exists('?pg', $urlNavigationArray))
					{
						$urlNavigationArray['?pg'] = $i;

						$output = implode('&', array_map(
						function ($v, $k) {
							if(is_array($v)){
								return $k.'[]='.implode('&'.$k.'[]=', $v);
							}else{
								return $k.'='.$v;
							}
						},
						$urlNavigationArray,
						array_keys($urlNavigationArray)
						));

						$url=$this->url_domain.$this->surl.$output;

						$response = $this->executeCurl($url);

						if(substr_count($response,'No torrents found'))
						{
							return;
						}
						else
						{
							$this->processResultList($plugin, $response);
						}
					}
				}
			} catch (Exception $e) {
			}
		}

		/* Begin private methods */

		private function configureCurl($curl, $url)
		{
			$headers = array
				(
					'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*;q=0.8',
					'Accept-Language: en-us;q=0.7,en;q=0.3',
					'Accept-Encoding: deflate',
					'Accept-Charset: windows-1251,utf-8;q=0.7,*;q=0.7'
				);

				curl_setopt($curl, CURLOPT_HTTPHEADER,$headers);
				curl_setopt($curl, CURLOPT_URL, $url);
				curl_setopt($curl, CURLOPT_FAILONERROR, 1);
				curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
				curl_setopt($curl, CURLOPT_RETURNTRANSFER,1);
				curl_setopt($curl, CURLOPT_TIMEOUT, 120);
				curl_setopt($curl, CURLOPT_USERAGENT, DOWNLOAD_STATION_USER_AGENT);
				curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
				curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		}

		private function executeCurl($url)
		{
			$curl = curl_init();
			$this->configureCurl($curl, $url);
			$content = curl_exec($curl);
			curl_close($curl);
			return $content;
		}

		private function processResultList($plugin, $response)
		{
			$regex_recordMagnetLink = 'href="(magnet.*)"';
			$regex_bitTorrentInfoHash = 'btih:(.*)&';

			if(preg_match_all("/$regex_recordMagnetLink/Us", $response, $recordDetails, PREG_SET_ORDER))
			{
				foreach ($recordDetails as $recordDetail)
				{
					if(preg_match_all("/$regex_bitTorrentInfoHash/Us", $recordDetail[1], $hashes, PREG_SET_ORDER))
					{
						foreach ($hashes as $hash)
						{
							$time_start = microtime(true);

							// Api call
							$url = sprintf($this->aurl, $hash[1]);

							try {
								$apiResponse = $this->executeCurl($url);
								$recordToAdd = $this->createItem($apiResponse, $recordDetail[1], $hash[1]);
								$this->addItemToPlugin($plugin, $recordToAdd);
							} catch (Exception $e) {
							}

							// Api calls are limited by one per second...
							$time_end = microtime(true);
							$time = $time_end - $time_start;
							$timeToSleep = 1100000 - $time; // Take 1.5 seconds at the minimum
							usleep($timeToSleep);
						}
					}
				}
			}
		}

		private function createItem($response, $magnet, $hash)
		{
			$recordToAdd = array();
			$regex_apiAllFields = 'name":"(.*)",.*category":"(.*)",.*"url":"(.*)".*,.*seeders":(.*),.*leechers":(.*),.*size":(.*),.*time_indexed":(.*)(,|})';
			$regex_apiRequiredFields = 'name":"(.*)",.*category":"(.*)",.*"url":"(.*)".*time_indexed":(.*)(,|})';
			$regex_apiSeeders = 'seeders":(.*),';
			$regex_apiLeechers = 'leechers":(.*),';
			$regex_apiSize = 'size":(.*),';

			if(preg_match_all("/$regex_apiAllFields/Us", $response, $apiData, PREG_SET_ORDER))
			{
				foreach ($apiData as $apiInfo)
				{
					$recordToAdd["page"] = $apiInfo[3]; //1. url
					$recordToAdd["seeds"] = $apiInfo[4]; //2. seeds
					$recordToAdd["leechs"] = $apiInfo[5]; //3. leechs
					$recordToAdd["datetime"] = date('Y-m-d', $apiInfo[7]); //4. date
					$recordToAdd["title"] = $apiInfo[1]; //5. title
					$recordToAdd["download"] = $magnet; //6. magnet
					$recordToAdd["category"] = $apiInfo[2]; //7. category
					$recordToAdd["size"] = $apiInfo[6]; //8. size
					$recordToAdd["hash"] = $hash; //9. hash
				}
			}
			else if(preg_match_all("/$regex_apiRequiredFields/Us", $response, $apiData, PREG_SET_ORDER))
			{
				foreach ($apiData as $apiInfo)
				{
					$recordToAdd["page"] = $apiInfo[3]; //1. url
					$recordToAdd["seeds"] = 0; //2. seeds
					$recordToAdd["leechs"] = 0; //3. leechs
					$recordToAdd["datetime"] = date('Y-m-d', $apiInfo[4]); //4. date
					$recordToAdd["title"] = $apiInfo[1]; //5. title
					$recordToAdd["download"] = $magnet; //6. magnet
					$recordToAdd["category"] = $apiInfo[2]; //7. category
					$recordToAdd["size"] = 0; //8. size
					$recordToAdd["hash"] = $hash; //9. hash
				}

				if(preg_match_all("/$regex_apiSeeders/Us", $response, $seeders, PREG_SET_ORDER))
				{
					foreach ($seeders as $seeds)
						$recordToAdd["seeds"] = $seeds[1]; //2. seeds
				}

				if(preg_match_all("/$regex_apiLeechers/Us", $response, $leechers, PREG_SET_ORDER))
				{
					foreach ($leechers as $leechs)
						$recordToAdd["leechs"] = $leechs[1]; //3. leechs
				}

				if(preg_match_all("/$regex_apiSize/Us", $response, $leechers, PREG_SET_ORDER))
				{
					foreach ($leechers as $leechs)
						$recordToAdd["size"] = $apiSize[1]; //8. size
				}
			}
			return $recordToAdd;
		}

		private function addItemToPlugin($plugin, $recordToAdd)
		{
			if (array_key_exists('title', $recordToAdd) && strlen($recordToAdd["title"]) > 0)
			{
				$plugin->addResult($recordToAdd["title"],
					$recordToAdd["download"],
					(float)$recordToAdd["size"],
					date('Y-m-d',strtotime(str_replace("'", "", $recordToAdd["datetime"]))),
					$recordToAdd["page"],
					$recordToAdd["hash"],
					(int)$recordToAdd["seeds"],
					(int)$recordToAdd["leechs"],
					$recordToAdd["category"]);
			}
		}

		/* End private methods */
}
?>